﻿-- Author      : Nogarder
-- Create Date : 2012/3/16 15:38:56
--global
local floor = floor
local cos=cos
local sin=sin
local abs=abs
local pairs=pairs
local table_getn=table.getn
local NPA_GetTime=NPA.GetTime
local NPA_RunFunc=NPA.RunFunc
--
local NPA_SettingManager_GetResources=NPA.SettingManager.GetResources
--main
NPA.TextureManager={}
-----------------------------------------------------------------------------------------------
-----------------macro for rotation--------------------------------
local NPA_TextureManager_Rotation=function(x,y,angle)
	local ca=floor(cos(angle)*1000)/1000
	local sa=floor(sin(angle)*1000)/1000
	return x*ca-y*sa,x*sa+y*ca
end
--function for texture rotation
local NPA_TextureManager_SetTextureRotation=function(texture,left,right,top,bottom,angle)
	local centerx=(left+right)/2
	local centery=(top+bottom)/2

	local dleft=left-centerx
	local dright=right-centerx
	local dtop=top-centery
	local dbottom=bottom-centery
	
	local tlx,tly=NPA_TextureManager_Rotation(dleft,dtop,angle)
	local blx,bly=NPA_TextureManager_Rotation(dleft,dbottom,angle)
	local trx,try=NPA_TextureManager_Rotation(dright,dtop,angle)
	local brx,bry=NPA_TextureManager_Rotation(dright,dbottom,angle)

	tlx=tlx+centerx
	tly=tly+centery
	blx=blx+centerx
	bly=bly+centery
	trx=trx+centerx
	try=try+centery
	brx=brx+centerx
	bry=bry+centery

	texture:SetTexCoord(tlx,tly,blx,bly,trx,try,brx,bry)
end
--
NPA.TextureManager.SetTextureRotation=NPA_TextureManager_SetTextureRotation
--
--manage texture
NPA.TextureManager.ManageTexture=function(texture,data,scalex,scaley,noanchor,nocolor,offsetx,offsety)
	local texturedata=NPA_SettingManager_GetResources("Texture",data.ProfileName)

	texture:SetBlendMode(texturedata.Mode)

	if(not nocolor)then
		local color=texturedata.Color
		texture:SetVertexColor(color.R,color.G,color.B,color.A)
	end

	local angle=texturedata.RAngle
	if(angle==0)then
		texture:SetTexCoord(texturedata.Left,texturedata.Right,texturedata.Top,texturedata.Bottom)
	else
		NPA_TextureManager_SetTextureRotation(texture,texturedata.Left,texturedata.Right,texturedata.Top,texturedata.Bottom,angle)
	end

	local scalex=scalex or 1
	local scaley=scaley or 1
	local offsetx=offsetx or 0
	local offsety=offsety or 0

	texture:SetHeight(data.Height*scaley)
	texture:SetWidth(data.Width*scalex)

	texture:SetDrawLayer(data.Layer)

	if(not noanchor)then
		texture:ClearAllPoints()

		local anchordata=data.AnchorData
		texture:SetPoint(anchordata.From,texture:GetParent(),anchordata.To,anchordata.OffsetX*scalex+offsetx,anchordata.OffsetY*scaley+offsety)
	end

	texture:SetTexture(texturedata.Texture)
end
--manage texture2
NPA.TextureManager.ManageTexture2=function(texture,texturename)
	local texturedata=NPA_SettingManager_GetResources("Texture",texturename)

	texture:SetBlendMode(texturedata.Mode)
	local color=texturedata.Color
	texture:SetVertexColor(color.R,color.G,color.B,color.A)

	local angle=texturedata.RAngle
	if(angle==0)then
		texture:SetTexCoord(texturedata.Left,texturedata.Right,texturedata.Top,texturedata.Bottom)
	else
		NPA_TextureManager_SetTextureRotation(texture,texturedata.Left,texturedata.Right,texturedata.Top,texturedata.Bottom,angle)
	end
	texture:SetTexture(texturedata.Texture)
end
--manage texture3
NPA.TextureManager.ManageTexture3=function(texture,texturename,color)
	local texturedata=NPA_SettingManager_GetResources("Texture",texturename)

	texture:SetBlendMode(texturedata.Mode)
	local colordata=texturedata.Color
	texture:SetVertexColor(colordata.R*color.R,colordata.G*color.G,colordata.B*color.B,colordata.A*color.A)

	local angle=texturedata.RAngle
	if(angle==0)then
		texture:SetTexCoord(texturedata.Left,texturedata.Right,texturedata.Top,texturedata.Bottom)
	else
		NPA_TextureManager_SetTextureRotation(texture,texturedata.Left,texturedata.Right,texturedata.Top,texturedata.Bottom,angle)
	end
	texture:SetTexture(texturedata.Texture)
end
-----------------------------------------------------------
--------texture group-------------------------
--setcolor
NPA.TextureManager.SetTextureGroupColor=function(texturegroup,r,g,b,a)
	local textures=texturegroup.Textures
	for i=1,table_getn(textures)do
		if(textures[i]:IsShown())then
			textures[i]:SetVertexColor(r,g,b,a)
		end
	end
end
--manage
local NPA_TextureManager_ManageTexture=NPA.TextureManager.ManageTexture
NPA.TextureManager.ManageTextureGroup=function(texturegroup,setting)
	--font
	local i=0
	local textures=texturegroup.Textures
	for name,data in pairs(setting)do
		if(name~="Type")then
			i=i+1
			if(not textures[i])then
				textures[i]=texturegroup:CreateTexture()
			end
			NPA_TextureManager_ManageTexture(textures[i],data)
			textures[i]:Show()
		end
	end
	texturegroup.Count=i
	for j=i+1,table_getn(textures) do
		textures[j]:Hide()
	end
	
	texturegroup:SetWidth((setting.Width or 1))
	texturegroup:SetHeight((setting.Height or 1))
	texturegroup.Ziped=nil
end
--create
NPA.TextureManager.CreateTextureGroup=function(mainframe)
	local texturegroup=CreateFrame("Frame",nil,mainframe)
	texturegroup:SetPoint("CENTER",mainframe,"CENTER")
	texturegroup:SetWidth(1)
	texturegroup:SetHeight(1)
	texturegroup.Parent=mainframe
	texturegroup.Count=0
	texturegroup.Textures={}
	texturegroup:EnableMouse(false)
	texturegroup:SetScript("OnHide",function(self)
		local textures=self.Textures
		for i=1,self.Count do
			textures[i]:Hide()
		end
	end)
	texturegroup:SetScript("OnShow",function(self)
		local textures=self.Textures
		for i=1,self.Count do
			textures[i]:Show()
		end
	end)
	return texturegroup
end
--
--
NPA.TextureManager.CreateIconFrame=function(mainframe)
	local textureframe=CreateFrame("Frame",nil,mainframe)
	textureframe:SetWidth(1)
	textureframe:SetHeight(1)
	textureframe:EnableMouse(false)
	--
	local icon=textureframe:CreateTexture()
	icon:SetPoint("TOPLEFT",textureframe,"TOPLEFT",0,0)
	icon:SetPoint("BOTTOMRIGHT",textureframe,"BOTTOMRIGHT",0,0)
	icon:Show()
	textureframe.Icon=icon
	--
	return textureframe
end
